#!/usr/bin/env python3
import common

import argparse
import logging
import sys
import time
from gitlab import (
    Gitlab,
    GitlabGetError,
    GitlabMRClosedError,
)

critical_error = (
    "This is a critical error! Please make sure to:\n"
    " 1. merge the above-mentioned merge request by hand as soon as possible\n"
    "    (this is important, otherwise following merge requests will get stuck, too)\n"
    " 2. examine why this has happened and fix it in the CI pipeline"
)


def accept_merge_request(project, mr, rebase=False, should_remove_source_branch=True):
    """Attempt to merge a merge request, rebase if necessary"""
    merged = False
    pipeline_pending = False

    while not merged:
        # Update merge request before trying to merge it in order to get the latest
        # pipeline status
        try:
            updated_mr = project.mergerequests.get(
                id=mr.iid, retry_transient_errors=True
            )
            mr = updated_mr
        except GitlabGetError as e:
            print("WARNING: Could not update merge request object: %s" % e)

        # Try to merge the merge request
        try:
            mr.merge(should_remove_source_branch=should_remove_source_branch)
            if pipeline_pending:
                print("")
            if mr.state == "merged":
                merged = True
            else:
                if mr.merge_error:
                    print("Merge error: %s" % mr.merge_error)
                else:
                    print("Merge reported success, but MR state is '%s'" % mr.state)
                return False, mr.sha

        except GitlabMRClosedError as e:
            # See HTTP error codes for merge requests here:
            # https://docs.gitlab.com/ce/api/merge_requests.html#accept-mr
            logging.debug("Error from gitlab: %d", e.response_code)

            if e.response_code == 405:
                # Not allowed (draft, closed, pipeline pending or failed)
                # Contrary to the documentation, this response is also issued when the
                # merge failed due to a merge conflict. See GitLab issue:
                # https://gitlab.com/gitlab-org/gitlab/-/issues/364102

                if mr.has_conflicts:
                    # Merge conflict, automatic rebase not possible
                    if pipeline_pending:
                        print("")
                    print("Merge not possible, has to be rebased manually")
                    return False, mr.sha

                # If pipeline is running, wait for completion
                if not mr.head_pipeline:
                    # No pipeline created yet
                    print("No pipeline created yet")
                    time.sleep(1)
                elif mr.head_pipeline["status"] in common.pending_states:
                    # Pipeline pending
                    if not pipeline_pending:
                        print("Waiting for pending pipeline", end="", flush=True)
                        pipeline_pending = True
                    print(".", end="", flush=True)
                    time.sleep(1)
                else:
                    # Merge failed due to some other reason
                    if pipeline_pending:
                        print("")
                    print("Merge not possible for unkown reason")
                    return False, mr.sha

            elif e.response_code == 406:
                # Merge conflict, automatic rebase is possible
                if pipeline_pending:
                    print("")
                    pipeline_pending = False
                print("Merge not possible, but branch can be automatically rebased")
                if not rebase:
                    return False, mr.sha
                print("Trying to rebase...")
                mr = common.rebase_merge_request(project, mr)
                if mr.merge_error:
                    print("ERROR: rebase not possible\n'%s'" % mr.merge_error)
                    sys.exit(critical_error)
                print("Sucessfully rebased")

            else:
                if pipeline_pending:
                    print("")
                print("ERROR: merge not possible: %s" % e)
                sys.exit(critical_error)

    return True, mr.sha


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--gitlab-url",
        help="""URL to the GitLab instance""",
        dest="gitlab_url",
        required=True,
    )
    parser.add_argument(
        "--token",
        help="""GitLab REST API private access token""",
        dest="token",
        required=True,
    )
    parser.add_argument(
        "--project",
        help="""name of the GitLab project""",
        dest="project",
        required=True,
    )
    parser.add_argument(
        "--merge-request",
        help="""id of the merge request""",
        dest="merge_request",
        required=True,
    )
    parser.add_argument(
        "--rebase",
        help="""attempt to automatically rebase merge request if necessary""",
        dest="rebase",
        action="store_true",
        required=False,
    )

    args, _ = parser.parse_known_args()

    gitlab = Gitlab(args.gitlab_url, private_token=args.token)
    project = common.get_project(gitlab, args.project)

    try:
        merge_request = project.mergerequests.get(
            id=args.merge_request, retry_transient_errors=True
        )
    except GitlabGetError as e:
        sys.exit("Could not get merge request: %s" % e)

    if accept_merge_request(project, merge_request, rebase=args.rebase):
        print("Successfully merged")
    else:
        sys.exit(1)


if __name__ == "__main__":
    main()
