#!/usr/bin/env python3
"""

Downloads the job artifacts of a given job

"""

import argparse
import logging
import sys
import os
import zipfile
import tempfile
import gitlab as gl

__author__ = "Jonas Höppner"
__email__ = "jonas.hoeppner@garz-fricke.com"

GITLAB_SERVER = "https://git.seco.com"

verbose = 0


def download_job_artifacts(gitlab, dest, job, project=None, extract=False):
    """Downloads the artifacts and stores them ar dest/job_id
    If extract is set, the downloaded zipfile is extracted and removed.
    Returns either the dest path or the zipfile name.
    """
    # Accept either a gitlab job object, or the project and the job id
    if isinstance(job, gl.v4.objects.jobs.ProjectJob):
        gl_job = job
    else:
        # Accept either an gitlab object, or name or id to specify a project
        if isinstance(project, gl.v4.objects.projects.Project):
            gl_project = project
        else:
            gl_project = gitlab.projects.get(project)
        gl_job = gl_project.jobs.get(job)

    dest = os.path.join(dest, str(gl_job.id))
    os.makedirs(dest)
    file_name = os.path.join(dest, "artifacts.zip")
    with open(file_name, "wb") as f:
        gl_job.artifacts(streamed=True, action=f.write)
    if not extract:
        return file_name

    zipfile.ZipFile(file_name).extractall(dest)
    os.remove(file_name)
    return dest


def download_job_artifact(gitlab, dest, path, job, project=None):
    """Downloads one file from artifacts and stores it at dest"""
    # Accept either a gitlab job object, or the project and the job id
    if isinstance(job, gl.v4.objects.jobs.ProjectJob):
        gl_job = job
    else:
        # Accept either an gitlab object, or name or id to specify a project
        if isinstance(project, gl.v4.objects.projects.Project):
            gl_project = project
        else:
            gl_project = gitlab.projects.get(project)
        gl_job = gl_project.jobs.get(job)

    with open(dest, "wb") as f:
        gl_job.artifact(path=path, streamed=True, action=f.write)

    return dest


def main(args):
    parser = argparse.ArgumentParser(description=__doc__, usage="%(prog)s [OPTIONS]")

    parser.add_argument(
        "--gitlab-url",
        help="""URL to the GitLab instance""",
        dest="gitlab_url",
        action="store",
        default=GITLAB_SERVER,
    )
    parser.add_argument(
        "--token",
        help="""GitLab REST API private access token""",
        dest="token",
        required=True,
    )
    parser.add_argument(
        "--project",
        action="store",
        dest="project",
        help="Specify the project by either by id or by path.",
        required=True,
    )
    parser.add_argument(
        "--job",
        action="store",
        dest="job",
        help="Specify the job by id.",
        required=True,
    )
    parser.add_argument(
        "--extract",
        action="store_true",
        dest="job",
        default=False,
        help="Specify if the artifacts should be extracted after download.",
    )
    parser.add_argument(
        "--dest",
        "--destination",
        action="store",
        dest="destination",
        default=None,
        help="Folder where the artifacts are stored in, a local tmpfolder is generated if omnitted.",
    )
    parser.add_argument(
        "--path",
        action="store",
        default=None,
        help="Path inside the artifacts, if set only one single file is downloaded instead of the complete artifacts.",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="count",
        dest="verbose",
        default=0,
        help="Increase verbosity.",
    )

    options = parser.parse_args(args)
    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)

    logging.debug(options)
    gitlab = gl.Gitlab(options.gitlab_url, private_token=options.token)

    if options.path is None:
        if options.destination is None:
            destination = tempfile.mkstemp()
        else:
            destination = options.destination

        filename = download_job_artifacts(
            gitlab, destination, options.job, options.project, extract=True
        )
        print("Downloaded artifacts for job {} to {}".format(options.job, filename))
    else:
        if options.destination is None:
            destination = tempfile.mkdtemp()
        else:
            destination = options.destination

        filename = download_job_artifact(
            gitlab, destination, options.path, options.job, options.project
        )
        print(
            "Downloaded {} for job {} to {}".format(options.path, options.job, filename)
        )


if __name__ == "__main__":
    main(sys.argv[1:])
