#!/usr/bin/env python3
#
# Class to simply generate a markdown file
#


class MarkDownText:
    def __init__(self, text, bold=False, italic=False):
        self.bold = bold
        self.italic = italic
        self.text = text

    @property
    def length(self):
        l = len(self.text)
        if self.italic:
            l += 2
        if self.bold:
            l += 4
        return l

    def render(self):
        out = self.text.splitlines()
        if len(out) == 0:
            return [""]
        for i in range(0, len(out)):
            if self.bold:
                out[i] = "**{}**".format(out[i])
            if self.italic:
                out[i] = "_{}_".format(out[i])
        return out

    def __str__(self):
        return __class__ + " " + self.text


class MarkDownTableCell(MarkDownText):
    def __init__(self, row, col, content, bold=False, italic=False):
        self.row = row
        self.col = col
        super().__init__(content, bold, italic)

    def __str__(self):
        return "MarkDownTableCell (" + self.row.row + ":" + self.col + ") " + self.text

    @property
    def alignment(self):
        return self.row.table.get_alignment(self.col)

    @property
    def width(self):
        return self.row.table.get_column_width(self.col)

    def render(self):
        c = super().render()[0]
        alignment = self.alignment
        width = self.width

        width -= 2
        if alignment == "c":
            c = c.center(width)
        elif alignment == "r":
            c = c.rjust(width)
        else:  # default 'l'
            c = c.ljust(width)
        c = " " + c + " "
        return [c]


class MarkDownTableRow:
    def __init__(self, table, row):
        self.cols = []
        self.table = table
        self.row = row
        self._current_col = 0  # which to fill next

    @property
    def length(self):
        return len(self.cols)

    def _set_content(self, content, col):
        if self.length <= col:
            missing = col + 1 - len(self.cols)
            self.cols += [None] * missing
        self.cols[col] = content

    def add_content(self, content, col=-1, bold=False, italic=False):
        if not isinstance(content, list):
            content = [content]
        if col >= 0:
            self._current_col = col
        for c in content:
            self._set_content(
                MarkDownTableCell(self, self._current_col, c, bold, italic),
                self._current_col,
            )
            self._current_col += 1

    def __str__(self):
        c = ""
        for i in self.cols:
            c += "| " + i.content + " |"
        return "MarkDownTableRow (" + self.length + " cols) " + c

    def render(self):
        output = "|"
        for c in self.cols:
            output += c.render()[0] + "|"
        return [output]


class MarkDownTable:
    def __init__(self):
        self.rows = []
        self.alignment = "l"
        self._current_row = 0
        self.column_width = []

    def set_alignment(self, alignment):
        """Set the alignment as a string like lccr with one char for each column"""
        self.alignment = alignment

    @property
    def length(self):
        return len(self.rows)

    def get_row(self, row=-1):
        if row == -1:
            row = self.length

        if self.length <= row:
            for i in range(self.length, row + 1):
                self.rows.append(MarkDownTableRow(self, i))
        return self.rows[row]

    def get_header(self):
        return self.get_row(0)

    def __str__(self):
        return "MarkDownTable (" + self.length + " rows)"

    def get_alignment(self, col):
        if col < len(self.alignment):
            return self.alignment[col]
        return self.alignment[-1]

    def get_column_width(self, col):
        if col < len(self.column_width):
            return self.column_width[col] + 2
        return 60

    def get_column_count(self):
        max_cols = 0
        for c in self.rows:
            if max_cols < c.length:
                max_cols = c.length
        return max_cols

    def _render_2nd_line(self):
        cnt = self.get_column_count()
        out = "|"
        for i in range(0, cnt):
            alignment = self.get_alignment(i)
            width = self.get_column_width(i)
            if alignment == "c":
                out += ":" + "-" * (width - 2) + ":"
            elif alignment == "r":
                out += "-" * (width - 1) + ":"
            else:  # default 'l'
                out += ":" + "-" * (width - 1)
            out += "|"
        return [out]

    def _evaluate_column_width(self):
        cnt = self.get_column_count()
        self.column_width = [0] * cnt
        for row in self.rows:
            for i in range(0, cnt):
                if i >= len(row.cols):
                    continue
                l = row.cols[i].length
                if l > self.column_width[i]:
                    self.column_width[i] = l

    def render(self):
        self._evaluate_column_width()
        text = [""]
        # Header line
        text += self.get_header().render()
        text += self._render_2nd_line()
        for row in self.rows[1:]:
            text += row.render()
        return text


class MarkDownHeader(MarkDownText):
    def __init__(self, text, level=0, bold=False, italic=False):
        self.level = level
        super().__init__(text, bold, italic)

    def render(self):
        out = "#" * (self.level + 1) + " " + super().render()[0]
        return ["", out, ""]


class MarkDownTextBlock(MarkDownText):
    def __init__(self, code, bold=False, italic=False, before=1, after=1):
        self.before = before
        self.after = after
        super().__init__(code, bold, italic)

    def render(self):
        out = []
        out += [""] * self.before
        out += super().render()
        out += [""] * self.after
        return out


class MarkDownCodeBlock(MarkDownText):
    def __init__(self, code):
        super().__init__(code)

    def render(self):
        out = []
        out += ["<pre>"]
        out += super().render()
        out += ["</pre>"]
        return out


class MarkDownAnchor:
    def __init__(self, name):
        self.name = name

    def render(self):
        out = '<a name="{}"></a>'.format(self.name)
        return [out]


class MarkDownDoc:
    def __init__(self):
        self.elements = []

    def add_element(self, MarkDownelement):
        self.elements.append(MarkDownelement)

    def add_table(self):
        table = MarkDownTable()
        self.add_element(table)
        return table

    def add_header(self, text=None, level=0):
        header = MarkDownHeader(text, level)
        self.add_element(header)
        return header

    def add_text_block(self, text=None, bold=False, italic=False, before=1, after=1):
        block = MarkDownTextBlock(text, bold, italic, before, after)
        self.add_element(block)
        return block

    def add_text(self, text=None, bold=False, italic=False):
        block = MarkDownText(text, bold, italic)
        self.add_element(block)
        return block

    def add_code_clock(self, code):
        block = MarkDownCodeBlock(code)
        self.add_element(block)
        return block

    def add_anchor(self, name):
        anchor = MarkDownAnchor(name)
        self.add_element(anchor)
        return anchor

    def add_collapse_block(self, title=None):
        collapse = MarkDownCollapseBlock(title)
        self.add_element(collapse)
        return collapse

    def render(self):
        out = []
        for e in self.elements:
            out += e.render()
        return out

    def to_string(self):
        return "\n".join(self.render())


class MarkDownCollapseBlock(MarkDownDoc):
    def __init__(self, title=None):
        self.title = title
        super().__init__()

    def render(self):
        out = []
        out += ["<details>"]
        if self.title is not None:
            out += ["<summary>{}</summary>".format(self.title)]
        out += super().render()
        out += ["</details>"]
        return out


def main():
    doc = MarkDownDoc()
    doc.add_header("Hello world")
    doc.add_text_block("This is the text")
    print("\n".join(doc.render()))


if __name__ == "__main__":
    main()
