#!/usr/bin/env python3
import common

import argparse
import logging
import sys
import time
from gitlab import Gitlab
from accept_merge_request import accept_merge_request
from create_merge_request import create_merge_request
from get_merge_requests import get_merge_requests
from integrate_into_manifest import integrate_into_manifest


def merge_into_manifest(
    manifest_project,
    manifest_branch,
    project,
    project_branch,
    srcrev_file,
    recipe_name,
    commit,
):
    """
    Create a merge request on the manifest for a given merged project commit and merge
    it immediately. Update the integration branch before, if it is not up to date with
    the current manifest master.
    """

    # Get source merge request
    mrs = get_merge_requests(
        project,
        target_branch=project_branch,
        state="merged",
        commit=commit,
    )
    if not mrs:
        sys.exit(
            "ERROR: could not determine source merge request for commit %s" % commit
        )
    source_mr = mrs[0]

    # Get original branch for commit
    original_branch = source_mr.source_branch
    logging.debug("Original branch %s", original_branch)

    if common.is_gitlab_ci_integration_branch(original_branch):
        print(
            "Commit {} is on a gitlab-ci integration branch and will be merged by the gitlab-ci pipeline.".format(
                commit
            )
        )
        return ""

    target_branch = manifest_branch

    integration_branch = common.integration_branch_name(
        project.name, original_branch, target_branch
    )

    # Create merge request. If there already is a merge request, we assume that it has
    # been opened manually and thus will be merged manually as well, so we fail here.
    mr, created = create_merge_request(
        manifest_project, integration_branch, target_branch
    )
    if not created:
        sys.exit("ERROR: There is already an open merge request:\n%s" % mr.web_url)

    print("Created new merge request:")
    print(mr.web_url)

    # Insert cross-links in both merge requests
    common.crosslink_merge_requests(source_mr, mr)

    # Loop until MR has been merged successfully
    manifest_revision = mr.sha
    merged = False
    while not merged:

        # Wait until GitLab has checked merge status
        common.wait_until_merge_status_is_set(manifest_project, mr)

        # Attempt to merge
        merged, _ = accept_merge_request(manifest_project, mr)

        if not merged:
            # Merge failed, reintegrate the source merge request into the manifest.
            # Note: if reintegration is necessary here, the source merge request was
            # merged without running the pipeline on the latest manifest, i.e. some
            # other project has been merged in between. Automatic rebase is not
            # possible, though, because the altered manifest lines are part of the
            # diff context, so we have to create a new integration commit. This
            # possibly might result in a failing pipeline state on the master. We
            # deliberately put up with this, as it is quite unlikely to happen and
            # we want a completely automated process in every case.
            manifest_revision = integrate_into_manifest(
                manifest_project=manifest_project,
                manifest_branch=target_branch,
                manifest_file=common.manifest_file,
                srcrev_file=srcrev_file,
                recipe_name=recipe_name,
                project=project,
                merge_request=source_mr,
            )

            # Wait until GitLab has updated the MR
            print("Waiting until MR has been updated", end="", flush=True)
            while mr.sha != manifest_revision:
                print(".", end="", flush=True)
                time.sleep(1)
                mr = manifest_project.mergerequests.get(
                    mr.iid, retry_transient_errors=True
                )
            print("")

    print("Successfully merged")

    return manifest_revision


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--gitlab-url",
        help="""URL to the GitLab instance""",
        dest="gitlab_url",
        required=True,
    )
    parser.add_argument(
        "--token",
        help="""GitLab REST API private access token""",
        dest="token",
        required=True,
    )
    parser.add_argument(
        "--manifest-project",
        help="""name of the manifest project""",
        dest="manifest_project",
        required=True,
    )
    parser.add_argument(
        "--manifest-branch",
        help="""manifest branch to merge changes into""",
        dest="manifest_branch",
        required=True,
    )
    parser.add_argument(
        "--project",
        help="""name of the project containing the commit to be merged""",
        dest="project",
        required=True,
    )
    parser.add_argument(
        "--project-branch",
        help="""project branch to merge changes into""",
        dest="project_branch",
        default=None,
        required=False,
    )
    parser.add_argument(
        "--srcrev-file",
        help="""source revision file name (default: 'SRCREV.conf')""",
        dest="srcrev_file",
        default=common.srcrev_file,
        required=False,
    )
    parser.add_argument(
        "--recipe-name",
        help="""recipe name to resolve the project in 'SRCREV.conf'""",
        dest="recipe_name",
        default=None,
        required=False,
    )
    parser.add_argument(
        "--commit",
        help="""sha of the commit to be merged""",
        dest="commit",
        required=True,
    )
    parser.add_argument(
        "--save-revision-to",
        help="""path to a file where the new manifest revision is stored""",
        dest="revision_file",
        required=False,
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help="""Increase verbosity.""",
    )

    args, _ = parser.parse_known_args()
    if args.verbose:
        logging.basicConfig(level=logging.DEBUG)

    gitlab = Gitlab(args.gitlab_url, private_token=args.token)
    project = common.get_project(gitlab, args.project)
    manifest_project = common.get_project(gitlab, args.manifest_project)

    # If no project_branch is set, assume manifest and project branches are the same
    if not args.project_branch:
        args.project_branch = args.manifest_branch

    manifest_revision = merge_into_manifest(
        manifest_project=manifest_project,
        manifest_branch=args.manifest_branch,
        project=project,
        project_branch=args.project_branch,
        srcrev_file=args.srcrev_file,
        recipe_name=args.recipe_name,
        commit=args.commit,
    )

    # Write new manifest revision to file for next pipeline stage
    if args.revision_file:
        with open(args.revision_file, "w", encoding="utf-8") as file:
            file.write(manifest_revision)


if __name__ == "__main__":
    main()
