#!/usr/bin/env python3
import common

import argparse
import os
import sys
import time
from gitlab import Gitlab, GitlabGetError
from gitlab.v4.objects import Project


def mirror_pipeline_result(project: Project, pipeline_id, job=None):
    """
    Get latest pipeline status for a given commit and ref.
    The ref can be negated with a preceding '^', e.g. '^master' finds a pipeline on any
    ref different from 'master'.
    """

    pipeline = project.pipelines.get(pipeline_id, retry_transient_errors=True)

    # Wait for pipeline termination
    print("Waiting for pipeline %s" % pipeline.web_url)
    terminated_states = ["success", "failed", "canceled", "skipped"]
    while pipeline.status not in terminated_states:
        print(".", end="", flush=True)
        time.sleep(1)
        pipeline = project.pipelines.get(pipeline.id, retry_transient_errors=True)
    print("")
    print("Result: %s" % pipeline.status, flush=True)

    # If we are running in a job environment and the upstream status is canceled,
    # explicitly cancel this job as well
    if job and pipeline.status == "canceled":
        time.sleep(5)
        job.cancel()

    # Else just reflect job status in return value
    return pipeline.status


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--gitlab-url",
        help="""URL to the GitLab instance""",
        dest="gitlab_url",
        required=True,
    )
    parser.add_argument(
        "--token",
        help="""GitLab REST API private access token""",
        dest="token",
        required=True,
    )
    parser.add_argument(
        "--project",
        help="""name of the GitLab project""",
        dest="project",
        required=True,
    )
    parser.add_argument(
        "--pipeline",
        help="""id of the pipeline to mirror the result of""",
        dest="pipeline",
        required=True,
    )

    args, _ = parser.parse_known_args()

    gitlab = Gitlab(args.gitlab_url, private_token=args.token)
    project = common.get_project(gitlab, args.project)

    # Check if we are running in a GitLab job context
    job_project_path = os.environ.get("CI_PROJECT_PATH")
    job_id = os.environ.get("CI_JOB_ID")
    if job_project_path and job_id:
        try:
            job_project = common.get_project(gitlab, job_project_path)
            job = job_project.jobs.get(id=job_id, retry_transient_errors=True)
        except GitlabGetError:
            print("WARNING: job %s not found in project %s" % (args.job, args.project))
            job = None

    status = mirror_pipeline_result(project, args.pipeline, job)

    if status != "success":
        sys.exit(1)

    sys.exit(0)


if __name__ == "__main__":
    main()
